﻿Function New-RandomPassword {
<#
.SYNOPSIS
    This advanced function ...

.DESCRIPTION
    ...

.PARAMETER Environment
    The Environment parameter is used to separate log files into their own directories. Additionally, the Environment parameter can be used to assist in making logical decisions where varying events occur inside the code based on the current environment.

.PARAMETER Log
    The Log parameter is used to indicate that the function should produce logging output. The valid parameter values for the Log parameter are ToScreen, ToScreenAndFile, and ToFile. This will display logging on the screen only, on the screen and in a locally stored file, or in a locally stored file only, respectively.

.EXAMPLE
    This example ...
    PS > <Example>
    
.NOTES
    Name: New-RandomPassword
    Author: <Name>
    Comments: --
    Last Edit: xx/xx/xxxx [1.0]
    Template Version 2.0
    Version 1.0
        - <VersionNotes> (or delete if no version notes)
    Version 1.1
        -
    Version 1.2
        -
        -
    Version 2.0
#>
    [CmdletBinding()]
    Param (
        [Parameter()]
        [ValidateRange(4,35)]
        [int]$Length = 10,

        [Parameter()]
        [ValidateSet('Number','Uppercase','Lowercase','Symbol')]
        [string[]]$CharacterType = ('Number','Uppercase','Lowercase','Symbol'),

        [Parameter()]
        [string[]]$UnusedCharacter,

        #region Template code for Param block.
        [Parameter()]
        [ValidateSet('ACC','DEV','NON-PRD','PRD','QA','STG','TST')]
        [string]$Environment,

        [Parameter()]
        [ValidateSet('ToFile','ToScreen','ToScreenAndFile')]
        [string]$Log
        #endregion
    )

    Begin {
        #region Template code for Begin block.
            #region Catch Verbose being used.
            If ($PSBoundParameters['Verbose']) {
                $Log = 'ToScreen'
            }
            #endregion

            #region Create Write-Verbose function, log directory, and log file.
            Switch ($Log) {
                {$Log -eq 'ToScreen'} {
                    #region Modify the VerbosePreference variable.
                    $VerbosePreference = 'Continue'
                    break
                    #endregion
                }
                {$Log -eq 'ToFile' -or $Log -eq 'ToScreenAndFile'} {
                    #region Determine the file log directory (according to $Environment).
                    $LogDirectory = "$($MyInvocation.MyCommand.Name)"
                    If ($Environment) {
                        $LogPath = "$env:SystemDrive\support\Logs\$LogDirectory\$($Environment.ToUpper())"
                    } Else {
                        $LogPath = "$env:SystemDrive\support\Logs\$LogDirectory"
                    } # End If-Else.
                    #endregion

                    #region Create the file log directory (if needed).
                    If (-Not(Test-Path -Path $LogPath)) {
                        [System.Void](New-Item -Path $LogPath -ItemType Directory)
                    } # End If.
                    #endregion

                    #region Create the log file path variable.
                    $FileName = "$(Get-Date -Format 'DyyyyMMddThhmmsstt.fffffff').txt"
                    $LogFilePath = "$LogPath\$FileName"
                    #endregion

                    #region Create Write-Verbose function.
                    If ($Log -eq 'ToFile') {
                        Function Write-Verbose {
                            Param ($Message)
                            "[$(Get-Date -Format G)]: $Message" | Out-File -FilePath $LogFilePath -Append
                        }

                    } ElseIf ($Log -eq 'ToScreenAndFile') {
                        $VerbosePreference = 'Continue'
                        Function Write-Verbose {
                            Param ($Message)
                            Microsoft.PowerShell.Utility\Write-Verbose -Message $Message
                            "[$(Get-Date -Format G)]: $Message" | Out-File -FilePath $LogFilePath -Append
                        }
                    } # End If-ElseIf.
                #endregion
                }
                'default' {}
            } # End Switch.
            #endregion

            #region Write informational details.
            $BlockLocation = '[INFO   ]'
            Write-Verbose -Message "$BlockLocation Invoking the $($MyInvocation.MyCommand.Name) $($MyInvocation.MyCommand.CommandType) on $(Get-Date -Format F)."
            Write-Verbose -Message "$BlockLocation Invoking user is ""$env:USERDOMAIN\$env:USERNAME"" on the ""$env:COMPUTERNAME"" computer."
            If ($Log -eq 'ToFile' -or $Log -eq 'ToScreenAndFile') {
                Write-Verbose -Message "$BlockLocation Storing log file as ""$LogFilePath."""
            }
            Foreach ($Parameter in $PSBoundParameters.GetEnumerator()) {
                Write-Verbose -Message "$BlockLocation Including the ""$($Parameter.Key)"" parameter with the ""$($Parameter.Value -join '","')"" $(If ($Parameter.Value.Count -gt 1) {'values'} Else {'value'})."
            }
            #endregion

            #region Write block location, 
            $BlockLocation = '[BEGIN  ]'
            Write-Verbose -Message "$BlockLocation Entering the Begin block [$($MyInvocation.MyCommand.CommandType): $($MyInvocation.MyCommand.Name)]."
            #endregion
        #endregion
    } # End Begin.

    Process {
        #region Template code for Process block.
            #region Write block location.
            $BlockLocation = '[PROCESS]'
            Write-Verbose -Message "$BlockLocation Entering the Process block [$($MyInvocation.MyCommand.CommandType): $($MyInvocation.MyCommand.Name)]."
            #endregion
        #endregion

        #region Create character set.
        Write-Verbose -Message "$BlockLocation Creating a random password with $Length characters."
        Write-Verbose -Message "$BlockLocation Building the allowable character set."
        Foreach ($Character in $CharacterType) {
            Switch ($Character) {
                'Number' {
                    # Numbers.
                    For ($a = 48; $a -le 57; $a++) {
                        $CharacterSet += [array][char]$a
                    }
                }
                'Uppercase' {
                    # Uppercase.
                    For ($b = 65; $b -le 89; $b++) {
                        $CharacterSet += [array][char]$b
                    }
                }
                'Lowercase' {
                    # Lowercase.
                    For ($c = 97; $c -le 122; $c++) {
                        $CharacterSet += [array][char]$c
                    }
                }
                'Symbol' {
                    # Symbols.
                    For ($d = 33; $d -le 47; $d++) {
                        $CharacterSet += [array][char]$d
                    }
                    For ($d = 58; $d -le 64; $d++) {
                        $CharacterSet += [array][char]$d
                    }
                    For ($d = 91; $d -le 96; $d++) {
                        $CharacterSet += [array][char]$d
                    }
                    For ($d = 123; $d -le 126; $d++) {
                        $CharacterSet += [array][char]$d
                    }    
                }
            } # End Switch.
        } # End Foreach.
        Write-Verbose -Message "$BlockLocation Displaying the allowable characters set as $($CharacterSet -join ',')."
        #endregion

        #region Remove unwanted characters.
        If ($UnusedCharacter) {
            Write-Verbose -Message "$BlockLocation Removing unwanted character(s) $($UnusedCharacter -join ',') from the character set."

            $CharacterSet = {$CharacterSet}.Invoke()
            Foreach ($Character in $UnusedCharacter) {
                [System.Void]($CharacterSet.Remove($Character))
            }
            Write-Verbose -Message "$BlockLocation Displaying the updated allowable characters set as $($CharacterSet -join ',')."
        } # End If.
        #endregion                                                                              s

        #region Create random password.
        Write-Verbose -Message "$BlockLocation Creating the random password."
        For ($Loop = 1; $Loop –le $Length; $Loop++) {
            $RandomPassword += ($CharacterSet | Get-Random)
        } # End For.
        #endregion.
    } # End Process.

    End {
        #region Template code for End block.
            #region Write block location.
            $BlockLocation = '[END    ]'
            Write-Verbose -Message "$BlockLocation Entering the End block [$($MyInvocation.MyCommand.CommandType): $($MyInvocation.MyCommand.Name)]."
            #endregion
        #endregion

        #region Display random password.
        Write-Verbose -Message "$BlockLocation Displaying the random password."
        $RandomPassword
        #endregion
    } # End End.
} # End Function: New-RandomPassword.

# The following commands are in place for the Arizona PowerShell Saturday 2017.
Clear-Host